<?php

declare(strict_types=1);

namespace mirzaev\arming_bot\controllers;

// Files of the project
use mirzaev\arming_bot\controllers\core,
	mirzaev\arming_bot\models\product,
	mirzaev\arming_bot\models\cart as model;

// Framework for ArangoDB
use mirzaev\arangodb\document;

/**
 * Controller of cart
 *
 * @package mirzaev\arming_bot\controllers
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class cart extends core
{
	/**
	 * Registry of errors
	 */
	protected array $errors = [
		'session' => [],
		'account' => [],
		'cart' => []
	];

	/**
	 * Write or delete from the cart
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function write(array $parameters = []): ?string
	{
		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			// POST request

			// The cart contains the product?
			$status = false;

			// Validating @todo add throwing errors
			if (!empty($parameters['product']) && preg_match('/[\d]+/', urldecode($parameters['product']), $matches)) $product = (int) $matches[0];

			if (isset($product)) {
				// Received and validated identfier of the product

				// Search for the product
				$product = product::read(
					filter: "d.identifier == @identifier && d.deleted != true && d.hidden != true",
					sort: 'd.created DESC',
					amount: 1,
					parameters: ['identifier' => $product],
					errors: $this->errors['cart']
				);

				if ($product instanceof product) {
					// Initialized the product

					// Initializing of the cart
					$cart = $this->session->cart(errors: $this->errors['cart']);

					if ($cart instanceof model) {
						// Initialized the cart

						if (0 < $amount = $cart->has($product, errors: $this->errors['cart'])) {
							// The cart contains the product

							// temporary
							/* $cart->disconnect($product, errors: $this->errors['cart']); */
							$cart->cancel($product, errors: $this->errors['cart']);

							$status = false;
						} else {
							// The cart not contains the product

							// temporary
							$cart->connect($product, errors: $this->errors['cart']);

							$status = true;
						}
					}
				}
			}

			// Initializing a response headers
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Initializing of the output buffer
			ob_start();

			// Generating the reponse
			echo json_encode(
				[
					'status' => $status ?? false,
					'errors' => $this->errors
				]
			);

			// Initializing a response headers
			header('Content-Length: ' . ob_get_length());

			// Sending and deinitializing of the output buffer
			ob_end_flush();
			flush();

			// Exit (success)
			return null;
		}

		// Exit (fail)
		return null;
	}
}
