"use strict";

// Import dependencies
import("/js/core.js").then(() =>
	import("/js/damper.js").then(() => {
		const dependencies = setInterval(() => {
			if (
				typeof core === "function" &&
				typeof core.damper === "function"
			) {
				clearInterval(dependencies);
				clearTimeout(timeout);
				initialization();
			}
		}, 10);
		const timeout = setTimeout(() => {
			clearInterval(dependencies);
			initialization();
		}, 5000);

		function initialization() {
			if (typeof core.cart === "undefined") {
				// Not initialized

				/**
				 * @name Cart
				 *
				 * @description
				 * Implements actions with cart
				 *
				 * @memberof core
				 *
				 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
				 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
				 */
				core.cart = class cart {
					/**
					 * Write or delete product from the cart  (interface)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {bool} True if an error occurs to continue the event execution
					 */
					static toggle(button, force = false) {
						// Blocking the button
						button.setAttribute("disabled", "true");

						// Execute under damper
						this.toggle_damper(button, force);

						// Exit (success)
						return false;
					}

					/**
					 * Write or delete product from the cart (damper)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {void}
					 */
					static toggle_damper = core.damper(
						(...variables) => this.toggle_system(...variables),
						800,
						1,
					);

					/**
					 * Write or delete product from the cart (system)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 *
					 * @return {Promise} Request to the server
					 *
					 * @todo add unblocking button by timer + everywhere
					 */
					static async toggle_system(button) {
						if (button instanceof HTMLElement) {
							// Validated

							// Initializing of identifier of the product
							const identifier = button.getAttribute("data-product-identifier");

							if (typeof identifier === "string" && identifier.length > 0) {
								// Validated identifier

								return await core.request(
									"/cart/write",
									`product=${identifier}`,
								)
									.then((json) => {
										if (
											json.errors !== null &&
											typeof json.errors === "object" &&
											json.errors.length > 0
										) {
											// Fail (received errors)
										} else {
											// Success (not received errors)

											// Unblocking the button
											button.removeAttribute("disabled");

											if (json.status) {
												// The product in the cart

												// Writing style of added to the cart button
												button.classList.add("cart");
											} else {
												// The product is not in the cart

												// Deleting style of added to the cart button
												button.classList.remove("cart");
											}
										}
									});
							}
						}
					}
				};
			}
		}
	})
);
